// TinyBASIC program, Authored by Mike Field
// Ported from the Arduino to the 8051 by Valentin Angelovski 
// Source modified to compile under the fleatiny kernel using SDCC


__data unsigned char 	linelen = 0;


// ASCII Characters
#define CR		'\r'
#define NL		'\n'
#define TAB		'\t'
#define BELL	'\b'
#define SPACE   ' '
#define CTRLC	0x03
#define CTRLH	0x08
#define CTRLS	0x13
#define CTRLX	0x18



/***********************************************************/
// Keyword table and constants - the last character has 0x80 added to it
const unsigned char keywords[] = {
	'L','I','S','T'+0x80,
	'L','O','A','D'+0x80,
	'N','E','W'+0x80,
	'R','U','N'+0x80,
	'S','A','V','E'+0x80,
	'N','E','X','T'+0x80,
	'L','E','T'+0x80,
	'I','F'+0x80,
	'G','O','T','O'+0x80,
	'G','O','S','U','B'+0x80,
	'R','E','T','U','R','N'+0x80,
	'R','E','M'+0x80,
	'F','O','R'+0x80,
	'I','N','P','U','T'+0x80,
	'P','R','I','N','T'+0x80,
	'P','O','K','E'+0x80,
	'E','N','D'+0x80,
	'B','Y','E'+0x80,
	'F','R','E','E'+0x80,
	'C','L','S'+0x80,
	'D','E','L','A','Y'+0x80,	
	0
};

#define KW_LIST		0
#define KW_LOAD		1
#define KW_NEW		2
#define KW_RUN		3
#define KW_SAVE		4
#define KW_NEXT		5
#define KW_LET		6
#define KW_IF		7
#define KW_GOTO		8
#define KW_GOSUB	9
#define KW_RETURN	10
#define KW_REM		11
#define KW_FOR		12
#define KW_INPUT	13
#define KW_PRINT	14
#define KW_POKE		15
#define KW_END		16
#define KW_BYE		17
#define KW_MEM		18
#define KW_CLS		19
#define KW_DELAY	20
#define KW_DEFAULT	21

struct stack_for_frame {
	char frame_type;
	char for_var;
	short int terminal;
	short int step;
	unsigned char *current_line;
	unsigned char *txtpos;
};

struct stack_gosub_frame {
	char frame_type;
	unsigned char *current_line;
	unsigned char *txtpos;
};

const unsigned char func_tab[] = {
	'P','E','E','K'+0x80,
	'A','B','S'+0x80,
	0
};
#define FUNC_PEEK    0
#define FUNC_ABS	 1
#define FUNC_UNKNOWN 2

const unsigned char to_tab[] = {
	'T','O'+0x80,
	0
};

const unsigned char step_tab[] = {
	'S','T','E','P'+0x80,
	0
};

const unsigned char relop_tab[] = {
	'>','='+0x80,
	'<','>'+0x80,
	'>'+0x80,
	'='+0x80,
	'<','='+0x80,
	'<'+0x80,
	0
};

#define RELOP_GE		0
#define RELOP_NE		1
#define RELOP_GT		2
#define RELOP_EQ		3
#define RELOP_LE		4
#define RELOP_LT		5
#define RELOP_UNKNOWN	6

#define STACK_SIZE (sizeof(struct stack_for_frame)*5)
#define VAR_SIZE sizeof(short int) // Size of variables in bytes



static const unsigned char okmsg[]		= "OK";
static const unsigned char sderrormsg[]	= "Drive fail";
static const unsigned char whatmsg[]	= "What? ";
static const unsigned char redomsg[]	= "Redo";
static const unsigned char howmsg[]		= "How?";
static const unsigned char sorrymsg[]	= "Sorry!";
static const unsigned char initmsg[]	= "TinyBasic in C V0.03.";
static const unsigned char valmsg[]		= "8051 port by Valentin Angelovski";
static const unsigned char memorymsg[]	= " bytes free.";
static const unsigned char breakmsg[]	= "break!";
static const unsigned char unimplimentedmsg[]	= "Unimplemented";
static const unsigned char backspacemsg[]		= "\b \b";

void TEXTMODE(void);
void tty_inchar(void); 
void tty_outchar(void); 
static void line_terminator(void);
static short int expression(void);
static unsigned char breakcheck(void);
void userprg(void); 


typedef unsigned short LINENUM;
LINENUM linenum;

unsigned char *txtpos,*inputpos,*list_line;
unsigned char expression_error;
unsigned char *tempsp;


unsigned char *stack_limit;
unsigned char *program_start;
unsigned char *program_end;
unsigned char *stack; // Software stack for things that should go on the CPU stack
unsigned char *variables_begin;
unsigned char *current_line;
unsigned char *sp;

#define STACK_GOSUB_FLAG 'G'
#define STACK_FOR_FLAG 'F'
unsigned char table_index;
__xdata __at (0x0200) unsigned char program[512];
